/*
 * System Interface Library for games
 * Copyright (c) 2007-2014 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/test/random.c: Tests for the random number generation functions.
 */

#ifdef INCLUDE_TESTS  // To the end of the file.

#include "src/base.h"
#include "src/random.h"
#include "src/test/base.h"

/*************************************************************************/
/*************************************************************************/

DEFINE_GENERIC_TEST_RUNNER(test_random)

/*-----------------------------------------------------------------------*/

TEST(test_initial_seed)
{
    CHECK_INTEQUAL(urandom64(), 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_srandom)
{
    srandom(1);
    CHECK_INTEQUAL(urandom64(), 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_srandom64)
{
    srandom64(1);
    CHECK_INTEQUAL(urandom64(), 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_srandom_env)
{
    srandom64(1);
    srandom_env();
    CHECK_TRUE(urandom64() != 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_random_)  // Avoid name collision with test runner.
{
    srandom64(1);
    CHECK_INTEQUAL(random(), 0x2C28FA16);
    CHECK_TRUE(random() != 0x2C28FA16);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_random_r)
{
    uint64_t state = 1;
    CHECK_INTEQUAL(random_r(&state), 0x2C28FA16);
    CHECK_INTEQUAL(state, 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_urandom)
{
    srandom64(1);
    CHECK_INTEQUAL(urandom(), 0x5851F42D);
    CHECK_TRUE(urandom() != 0x5851F42D);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_urandom_r)
{
    uint64_t state = 1;
    CHECK_INTEQUAL(urandom_r(&state), 0x5851F42D);
    CHECK_INTEQUAL(state, 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_random64)
{
    srandom64(1);
    CHECK_INTEQUAL(random64(), 0x2C28FA16A64ABF97LL);
    CHECK_TRUE(random64() != 0x2C28FA16A64ABF97LL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_random64_r)
{
    uint64_t state = 1;
    CHECK_INTEQUAL(random64_r(&state), 0x2C28FA16A64ABF97LL);
    CHECK_INTEQUAL(state, 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_urandom64)
{
    srandom64(1);
    CHECK_INTEQUAL(urandom64(), 0x5851F42D4C957F2EULL);
    CHECK_TRUE(urandom64() != 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_urandom64_r)
{
    uint64_t state = 1;
    CHECK_INTEQUAL(urandom64_r(&state), 0x5851F42D4C957F2EULL);
    CHECK_INTEQUAL(state, 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_frandom)
{
    srandom64(1);
    CHECK_DOUBLEEQUAL(frandom(), 0.34500051599441939);
    CHECK_TRUE(frandom() != 0.34500051599441939);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_frandom_r)
{
    uint64_t state = 1;
    CHECK_DOUBLEEQUAL(frandom_r(&state), 0.34500051599441939);
    CHECK_INTEQUAL(state, 0x5851F42D4C957F2EULL);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_frandomf)
{
    srandom64(1);
    CHECK_FLOATEQUAL(frandomf(), 0.34500051f);
    CHECK_TRUE(frandomf() != 0.34500051f);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_frandomf_r)
{
    uint64_t state = 1;
    CHECK_FLOATEQUAL(frandomf_r(&state), 0.34500051f);
    CHECK_INTEQUAL(state, 0x5851F42D4C957F2EULL);
    return 1;
}

/*************************************************************************/
/*************************************************************************/

#endif  // INCLUDE_TESTS

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
